/****************************************************************************
 *  pptpnapi.h    : $Revision:   1.9  $
 *  Modified      : $Date:   May 13 1996 14:48:42  $
 *  By            : $Author:   tmortsol  $, U.S. Robotics Access Corp.
 *****************************************************************************
 *          Copyright (c) $Date:   May 13 1996 14:48:42  $,                  *
 *            U.S. Robotics Access Corp., Microsoft Corp.                    *
 *                       All Rights Reserved                                 *
 *****************************************************************************
 *                          "PPTP MODULE"                                    *
 * This software may be used to implement the POINT to POINT TUNNELING       *
 * PROTOCOL as defined by the PPTP forum.  The PPTP forum consists of:       *
 * U.S. Robotics, Microsoft, Ascend Communications, 3Com/Primary Access,     *
 * Telematics.                                                               *
 *****************************************************************************
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain this entire header
 *    including the above copyright notice, this list of conditions and
 *    the following disclaimer.
 * 2. The name of U.S. Robotics may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission of U.S. Robotics.
 *
 * THIS SOFTWARE IS PROVIDED BY U.S. ROBOTICS ACESS CORP. ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL U.S. ROBOTICS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*END_HEADER*/

/*
 * pptpnapi.h
 *
 * This file contains prototypes for the network API routines
 * required by the PPTP protcol engine for both the PPTP client
 * and PPTP server implementations.  These routines interface
 * with a portability library on both the client and
 * server that in effect interface the networking portion of the PPTP
 * protocol engine into the networking environment of the target.
 * The API is divided into three areas:
 *
 *  - a set of routines required by the PPTP protocol engine to cause
 *    specific network events to occur.  These routines must be provided
 *    by the portability library on the target.
 *
 *  - a set of routines provided by the PPTP protocol engine
 *    that must be called by portability library on the target when
 *    certain network events occur.
 *
 * - a set of memory allocation routines required by the PPTP protocol
 *   engine.  Currently these are unused
 *
 * - a set of timer routines required by the PPTP protocol engine.  These
 *   routines must be provided by the portability library on the target.
 *
 * The API routines are generalized so they can work on both a Windows
 * NT client and our device chassis.  It should be noted that these
 * routines may be general enough to port to other environments but this
 * is not guaranteed.
 */

#if !defined(_pptpnapi_h)
#define _pptpnapi_h

/*
 * The following network routines are called by the PPTP protocol engine
 * to perform data and control channel operations.  An implementation of
 * PPTP client/server must provide these routines and the functionality
 * they require.
 */

/*
 * PptpNetControlConnectReq()
 *
 * This routine initiates a control channel connection to a remote
 * PPTP protocol engine.
 */

PptpReturnCode PptpNetControlConnectReq(
			     PptpSockaddrIn *remoteAddress,
			     PptpEndpointHandle *endpointHandle
			     );

/*
 * PptpNetControlListenReq()
 *
 *
 * This routine causes a control channel to beginning listening and accepting
 * incoming control channel connections on a specified port number.
 */

PptpReturnCode PptpNetControlListenReq(
    PptpPortNumber		portNumber,
    PptpEndpointHandle		*endpointHandle
    );

/*
 * PptpNetControlDisconnectReq()
 *
 * This routine initiates a shutdown of a control channel.  This routine
 * can be called during any protocol state of a control connection.
 */

PptpReturnCode PptpNetControlDisconnectReq(
	PptpEndpointHandle	endpointHandle,
	BOOL				AbortiveDisconnect
	);

/*
 * PptpNetGREGetHandle()
 *
 * This routine begins listening on GRE port for a specified protocol
 * number and obtains a handle for that GRE port that can be used to
 * send to a remote endpoint
 */

PptpReturnCode PptpNetGREGetHandle(
	PptpProtocolNumber	protocolNumber,
	PptpEndpointHandle	*endpointHandle
	);

/*
 * PptpNetControlSendDataReq()
 *
 * This routine intitiates a send of a sequence of control data to the
 * remote endpoint.
 */

PptpReturnCode PptpNetControlSendDataReq(
	PptpEndpointHandle	endpointHandle,
	UCHAR			*data,
	ULONG			dataLength,
	ULONG			*dataLengthSent
	);

/*
 * PptpNetDataChannelSendDataReq()
 *
 * This routine intitiates a send of a data channel packet to the
 * remote endpoint
 */

PptpReturnCode PptpNetDataChannelSendDataReq(
	PptpEndpointHandle	endpointHandle,
	PptpDataDescriptor	*dataDescriptor,
	PptpSockaddrIn		*remoteAddress,
	PptpProtocolNumber	remoteProtocolNumber
	);

/*
 * PptpNetDataChannelRecvDataCompleteReq()
 *
 * This routine is called by the PPTP protocol engine to indicate is
 * is done processing a specific received data channel data buffer
 */

PptpReturnCode PptpNetDataChannelRecvDataCompleteReq(
	void			*completionArg
	);

/*
 * The following network callback routines are provided by the PPTP protocol
 * engine to handle specific PPTP network events.  An implementation of
 * the PPTP client/server must call these routines when the corresponding
 * events occur.
 */


/*
 * PptpNetControlConnectEstablishedInd()
 *
 * This routine is called to indicate that a requested control connection
 * has been established and accepted by the remote endpoint.
 */

PptpReturnCode PptpNetControlConnectEstablishedInd(
	PptpEndpointHandle	endpointHandle
	);

/*
 * PptpNetControlNewIncomingConnectionInd()
 *
 * This routine is called to indicate that an incoming control connection
 * has arrived and been accepted.
 */

PptpReturnCode PptpNetControlIncomingConnectionInd(
	PptpEndpointHandle	endpointHandle,
	PptpSockaddrIn		*remoteAddress
	);

/*
 * PptpNetControlDisconnectInd()
 *
 * This routine is called to indicate that an existing control connection
 * has disconnected.  This can occur during any protocol state of a control
 * connection.
 */

PptpReturnCode PptpNetControlDisconnectInd(
	PptpEndpointHandle	endpointHandle
	);

/*
 * PptpNetControlRecvDataInd()
 *
 * This routine is upon reception of incoming data from a remote endpoint
 * within the control channel.
 */

void PptpNetControlRecvDataInd(
	PptpEndpointHandle	endpointHandle,
	UCHAR			*data,
	ULONG			dataLength
	);

/*
 * PptpNetDataChannelRecvDataInd()
 *
 * This routine is called upon reception of an incoming device packet
 * from the remote endpoint.
 */

PptpReturnCode PptpNetDataChannelRecvDataInd(
	PptpSockaddrIn		*remoteAddr,
	PptpProtocolNumber	remoteProtocolNumber,
	PptpDataDescriptor	*dataDescriptorp
	);

/*
 * PptpNetDataChannelSendDataCompleteInd()
 *
 * This routine is called to notify the PPTP engine that the handling of
 * an outbound packet for a remote endpoint has been completed and its
 * associated data buffer is no longer required.
 */

PptpReturnCode PptpNetDataChannelSendDataCompleteInd(
	void 			*completionArg
	);

/*
 * PptpAllocateSystemMemory()
 *
 * This routine is called by the PPTP protocol engine to allocate memory.
 * This routine is not currently used but will be used in the future to
 * dynamically allocate internal PPTP protocol engine data structures
 * during PPTP initialization
 */

PptpReturnCode PptpAllocateSystemMemoryReq(
	ULONG			numberOfBytes,
	VOID			**allocatedMemory
	);

/*
 * PptpFreeSystemMemoryReq()
 *
 * This routine is called by the PPTP protocol engine to free allocated
 * system memory allocated by PptpAllocateSystemMemoryReq()
 */

PptpReturnCode PptpFreeSystemMemoryReq(
	VOID			*allocatedMemory
	);

/*
 * PptpCreateExternalTimerReq()
 *
 * This routine is called to create a OS timer.
 */

PptpReturnCode PptpCreateExternalTimerReq(
	PptpCallbackFunction	callbackFunction,
	PptpCallbackArgument	callbackArgument,
	PptpMsec		milliSeconds,
	PptpExternalTimerId	*timerId
	);

/*
 * PptpDeleteExternalTimerReq()
 *
 * This routine is called to destroy an existing OS timer.
 */

PptpReturnCode PptpDeleteExternalTimerReq(
	PptpExternalTimerId	timerId
	);

/*
 * PptpGetSystemTimeReq()
 *
 * This routine is called to retrieve the current system time
 */

PptpReturnCode PptpGetSystemTimeReq(
	PptpMsec		*pMilliSeconds
	);

#endif /* !defined(_pptpnapi_h) */
